@@ -1,8 +1,8 @@
 // Copyright (C) 2005, 2006 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: MittelmannDistCntrlNeumB.hpp 1707 2010-07-03 18:25:51Z stefan $
+// $Id: MittelmannDistCntrlNeumB.hpp 2005 2011-06-06 12:55:16Z stefan $
 //
 // Authors:  Andreas Waechter           IBM    2005-10-18
 //                  based on MyNLP.hpp
@@ -13,6 +13,12 @@
 #include "IpTNLP.hpp"
 #include "RegisteredTNLP.hpp"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#else
+#include "configall_system.h"
+#endif
+
 #ifdef HAVE_CMATH
 # include <cmath>
 #else
@@ -23,7 +29,15 @@
 # endif
 #endif
 
-#include <cstdio>
+#ifdef HAVE_CSTDIO
+# include <cstdio>
+#else
+# ifdef HAVE_STDIO_H
+#  include <stdio.h>
+# else
+#  error "don't have header file for stdio"
+# endif
+#endif
 
 using namespace Ipopt;
 
