@@ -1,8 +1,8 @@
 // Copyright (C) 2004, 2008 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: IpDefaultIterateInitializer.cpp 1219 2008-04-25 18:14:35Z andreasw $
+// $Id: IpDefaultIterateInitializer.cpp 2339 2013-06-22 10:21:26Z stefan $
 //
 // Authors:  Carl Laird, Andreas Waechter              IBM    2004-09-23
 
@@ -547,7 +547,7 @@ namespace Ipopt
       //    p_l->Axpy(1.0, *tiny_l);  we shouldn't need this
 
       // At this point, p_l is
-      //  min(bound_push * max(|x_L|,1), bound_frac*(x_U-x_L)  for components
+      //  min(bound_push * max(|x_L|,1), bound_frac*(x_U-x_L)) for components
       //                                                       with two bounds
       //  bound_push * max(|x_L|,1)                            otherwise
       // This is the margin we want to the lower bound
@@ -582,7 +582,7 @@ namespace Ipopt
       p_u->ElementWiseReciprocal();
       p_u->Axpy(1.0, *tiny_u);
       // At this point, p_l is
-      //  min(bound_push * max(|x_U|,1), bound_frac*(x_U-x_L)  for components
+      //  min(bound_push * max(|x_U|,1), bound_frac*(x_U-x_L)) for components
       //                                                       with two bounds
       //  bound_push * max(|x_U|,1)                            otherwise
       // This is the margin we want to the upper bound
@@ -607,6 +607,7 @@ namespace Ipopt
     else {
       DBG_ASSERT(bound_push == 0.);
       tmp_l = x_L.MakeNewCopy();
+      DBG_PRINT_VECTOR(1, "tmp_l33", *tmp_l);
       Px_L.TransMultVector(-1.0, *my_orig_x, 1.0, *tmp_l);
       tmp_l->ElementWiseMax(*zero_l);
       DBG_PRINT_VECTOR(1, "tmp_l3", *tmp_l);
