@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2006 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: IpStdAugSystemSolver.cpp 1292 2008-08-25 15:56:34Z andreasw $
+// $Id: IpStdAugSystemSolver.cpp 2472 2014-04-05 17:47:20Z stefan $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -18,7 +18,6 @@
 #include "IpTripletHelper.hpp"
 // ToDo: Remove above here
 
-// for sprintf
 #ifdef HAVE_CSTDIO
 # include <cstdio>
 #else
@@ -116,6 +115,8 @@ namespace Ipopt
     DBG_START_METH("StdAugSystemSolver::MultiSolve",dbg_verbosity);
     DBG_ASSERT(J_c && J_d && "Currently, you MUST specify J_c and J_d in the augmented system");
 
+    IpData().TimingStats().StdAugSystemSolverMultiSolve().Start();
+
     DBG_ASSERT(W_factor == 0.0 || W_factor == 1.0);
 
     Index nrhs = (Index)rhs_xV.size();
@@ -175,12 +176,13 @@ namespace Ipopt
       augrhs->SetComp(2, *rhs_cV[i]);
       augrhs->SetComp(3, *rhs_dV[i]);
       char buffer[16];
-      sprintf(buffer, "RHS[%2d]", i);
+      Snprintf(buffer, 15, "RHS[%2d]", i);
       augrhs->Print(Jnlst(), J_MOREVECTOR, J_LINEAR_ALGEBRA, buffer);
       augmented_rhsV[i] = GetRawPtr(augrhs);
     }
 
     augmented_system_->Print(Jnlst(), J_MATRIX, J_LINEAR_ALGEBRA, "KKT");
+#ifndef HAVE_MPI
     if (Jnlst().ProduceOutput(J_MOREMATRIX, J_LINEAR_ALGEBRA)) {
       // ToDo: remove below here - for debug only
       Index dbg_nz = TripletHelper::GetNumberEntries(*augmented_system_);
@@ -201,6 +203,7 @@ namespace Ipopt
       dbg_values = NULL;
       // ToDo: remove above here
     }
+#endif
 
     // Call the linear solver
     std::vector<SmartPtr<Vector> > augmented_solV(nrhs);
@@ -221,7 +224,7 @@ namespace Ipopt
       Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA, "Factorization successful.\n");
       for (Index i=0; i<nrhs; i++) {
         char buffer[16];
-        sprintf(buffer, "SOL[%2d]", i);
+        Snprintf(buffer, 15, "SOL[%2d]", i);
         augmented_solV[i]->Print(Jnlst(), J_MOREVECTOR, J_LINEAR_ALGEBRA,
                                  buffer);
       }
@@ -230,6 +233,7 @@ namespace Ipopt
       Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA, "Factorization failed with retval = %d\n", retval);
     }
 
+    IpData().TimingStats().StdAugSystemSolverMultiSolve().End();
     return retval;
   }
 
