@@ -1,17 +1,12 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
-// This code is published under the Common Public License.
+// This code is published under the Eclipse Public License.
 //
-// $Id: AmplTNLP.cpp 1245 2008-05-30 03:46:59Z andreasw $
+// $Id: AmplTNLP.cpp 2477 2014-04-08 09:57:37Z stefan $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
 #include "IpoptConfig.h"
-#ifdef HAVE_CONFIG_H
-# include "config_ipopt.h"
-#else
-# define PACKAGE_VERSION IPOPT_VERSION
-#endif
 
 #include "AmplTNLP.hpp"
 #include "IpDenseVector.hpp"
@@ -19,6 +14,16 @@
 #include "IpSymTMatrix.hpp"
 #include "IpBlas.hpp"
 
+#ifdef HAVE_CSTRING
+# include <cstring>
+#else
+# ifdef HAVE_STRING_H
+#  include <string.h>
+# else
+#  error "don't have header file for string"
+# endif
+#endif
+
 /* AMPL includes */
 #include "asl.h"
 #include "asl_pfgh.h"
@@ -99,17 +104,23 @@ namespace Ipopt
 
     // check the problem statistics (see Table 1 in AMPL doc)
     DBG_ASSERT(n_var > 0); // need some continuous variables
-    if (!allow_discrete && (nbv>0 || niv>0) ) {
-      jnlst_->Printf(J_WARNING, J_MAIN, "==> Warning: Treating %d binary and %d integer variables as continous.\n\n", nbv, niv);
+    if (!allow_discrete && (nbv>0 || niv>0 || nlvbi>0 || nlvci>0 || nlvoi>0) ) {
+      jnlst_->Printf(J_WARNING, J_MAIN, "==> Warning: Treating %d binary and %d integer variables as continous.\n\n", nbv, niv+nlvbi+nlvci+nlvoi);
       allow_discrete = true;
     }
     allow_discrete = true;
-    ASSERT_EXCEPTION(allow_discrete || (nbv == 0 && niv == 0),
+    ASSERT_EXCEPTION(allow_discrete || (nbv == 0 && niv == 0 && nlvbi == 0 && nlvci == 0 && nlvoi == 0),
                      IpoptException,
                      "Discrete variables not allowed when the allow_discrete flag is false, "
                      "Either remove the integer variables, or change the flag in the constructor of AmplTNLP"
                     );
 
+    if (n_cc != 0) {
+      jnlst_->Printf(J_ERROR, J_MAIN, "\n\n***** Currently, the AMPL interface to Ipopt does not support\n      complementarity constraints.  Aborting. *****\n\n");
+      ASSERT_EXCEPTION(n_cc == 0, IpoptException,
+		       "Currently, the AMPL interface to Ipopt does not support complementarity constraints.");
+    }
+
     DBG_ASSERT(nlo == 0 || nlo == 1); // Can handle nonlinear obj.
     DBG_ASSERT(nwv == 0); // Don't know what "linear arc" variables are
     DBG_ASSERT(nlnc == 0); // Don't know what "nonlinear network"constraints are
@@ -297,6 +308,52 @@ namespace Ipopt
     return true;
   }
 
+  bool AmplTNLP::get_var_con_metadata(Index n,
+                                      StringMetaDataMapType& var_string_md,
+                                      IntegerMetaDataMapType& var_integer_md,
+                                      NumericMetaDataMapType& var_numeric_md,
+                                      Index m,
+                                      StringMetaDataMapType& con_string_md,
+                                      IntegerMetaDataMapType& con_integer_md,
+                                      NumericMetaDataMapType& con_numeric_md)
+  {
+    ASL_pfgh* asl = asl_;
+    DBG_ASSERT(asl_);
+
+    // pick up the variable and constraints names if available
+    Index rlen = maxrownamelen;
+    Index clen = maxcolnamelen;
+
+    if (rlen > 0) {
+      std::vector<std::string> var_names(n);
+      for (Index i=0; i<n; i++) {
+        var_names[i] = var_name(i);
+      }
+      var_string_md_["idx_names"] = var_names;
+    }
+
+    if (clen > 0) {
+      std::vector<std::string> con_names(m);
+      for (Index i=0; i<m; i++) {
+        con_names[i] = con_name(i);
+      }
+      con_string_md_["idx_names"] = con_names;
+    }
+
+    if (var_string_md_.size() > 0 || var_integer_md_.size() > 0 || var_numeric_md_.size() > 0
+        || con_string_md_.size() > 0 || con_integer_md_.size() > 0 || con_numeric_md_.size() > 0) {
+      var_string_md = var_string_md_;
+      var_integer_md = var_integer_md_;
+      var_numeric_md = var_numeric_md_;
+      con_string_md = con_string_md_;
+      con_integer_md = con_integer_md_;
+      con_numeric_md = con_numeric_md_;
+      return true;
+    }
+
+    return false;
+  }
+
   bool AmplTNLP::get_bounds_info(Index n, Number* x_l, Number* x_u, Index m, Number* g_l, Number* g_u)
   {
     ASL_pfgh* asl = asl_;
@@ -344,16 +401,9 @@ namespace Ipopt
     DBG_ASSERT(n == n_var);
     DBG_ASSERT(m == n_con);
 
-    if (init_x) {
-      for (Index i=0; i<n; i++) {
-        if (havex0[i]) {
-          x[i] = X0[i];
-        }
-        else {
-          x[i] = 0.0;
-        }
-      }
-    }
+    if (init_x)
+      for (Index i=0; i<n; i++)
+        x[i] = havex0[i] ? X0[i] : Max(LUv[2*i], Min(LUv[2*i+1], 0.0));
 
     if (init_z) {
       // Modified for warm-start from AMPL
@@ -361,31 +411,14 @@ namespace Ipopt
       const double* zL_init = suffix_handler_->GetNumberSuffixValues("ipopt_zL_in", AmplSuffixHandler::Variable_Source);
       const double* zU_init = suffix_handler_->GetNumberSuffixValues("ipopt_zU_in", AmplSuffixHandler::Variable_Source);
       for (Index i=0; i<n; i++) {
-        if (zL_init) {
-          z_L[i]=zL_init[i];
-        }
-        else {
-          z_L[i] =1.0;
-        }
-        if (zU_init) {
-          z_U[i]=zU_init[i];
-        }
-        else {
-          z_U[i] =1.0;
-        }
+        z_L[i] = zL_init != NULL ?  obj_sign_ * zL_init[i] : 1.0;
+        z_U[i] = zU_init != NULL ? -obj_sign_ * zU_init[i] : 1.0;
       }
     }
 
-    if (init_lambda) {
-      for (Index i=0; i<m; i++) {
-        if (havepi0[i]) {
-          lambda[i] = pi0[i];
-        }
-        else {
-          lambda[i] = 0.0;
-        }
-      }
-    }
+    if (init_lambda)
+      for (Index i=0; i<m; i++)
+        lambda[i] = havepi0[i] ? -obj_sign_ * pi0[i] : 0.0;
 
     return true;
   }
@@ -515,7 +548,7 @@ namespace Ipopt
       DBG_ASSERT(k==nele_hess);
       return true;
     }
-    else if (!iRow & !jCol && values) {
+    else if (!iRow && !jCol && values) {
       if (!apply_new_x(new_x, n, x)) {
         return false;
       }
@@ -572,10 +605,22 @@ namespace Ipopt
     }
 
     IpBlasDcopy(n, x, 1, x_sol_, 1);
-    IpBlasDcopy(n, z_L, 1, z_L_sol_, 1);
-    IpBlasDcopy(n, z_U, 1, z_U_sol_, 1);
     IpBlasDcopy(m, g, 1, g_sol_, 1);
-    IpBlasDcopy(m, lambda, 1, lambda_sol_, 1);
+    if( obj_sign_ == -1.0 ) // maximization
+    {
+      for( int i = 0; i < n; ++i )
+        z_L_sol_[i] = -z_L[i];
+      IpBlasDcopy(n, z_U, 1, z_U_sol_, 1);
+      IpBlasDcopy(m, lambda, 1, lambda_sol_, 1);
+    }
+    else
+    {
+      IpBlasDcopy(n, z_L, 1, z_L_sol_, 1);
+      for( int i = 0; i < n; ++i )
+        z_U_sol_[i] = -z_U[i];
+      for( int i = 0; i < m; ++i )
+        lambda_sol_[i] = -lambda[i];
+    }
     obj_sol_ = obj_value;
 
     std::string message;
@@ -587,6 +632,10 @@ namespace Ipopt
       message = "Maximum Number of Iterations Exceeded.";
       solve_result_num = 400;
     }
+    else if (status == CPUTIME_EXCEEDED) {
+      message = "Maximum CPU Time Exceeded.";
+      solve_result_num = 401;
+    }
     else if (status == STOP_AT_TINY_STEP) {
       message = "Search Direction becomes Too Small.";
       solve_result_num = 500;
@@ -608,7 +657,7 @@ namespace Ipopt
       solve_result_num = 501;
     }
     else if (status == DIVERGING_ITERATES) {
-      message = "Iterates divering; problem might be unbounded.";
+      message = "Iterates diverging; problem might be unbounded.";
       solve_result_num = 300;
     }
     else {
@@ -623,7 +672,7 @@ namespace Ipopt
     }
 
     // Write the .sol file
-    message = " \nIpopt " PACKAGE_VERSION ": " + message;
+    message = " \nIpopt " IPOPT_VERSION ": " + message;
     write_solution_file(message.c_str());
   }
 
@@ -666,7 +715,6 @@ namespace Ipopt
       g = new double[m];
       allocated = true;
     }
-
     conval(const_cast<Number*>(x), g, (fint*)nerror_);
 
     if (allocated) {
@@ -898,6 +946,18 @@ namespace Ipopt
     }
   }
 
+
+  AmplOptionsList::AmplOption::AmplOption(const std::string ipopt_option_name,
+                                          AmplOptionType type,
+                                          const std::string description)
+      :
+      ipopt_option_name_(ipopt_option_name),
+      type_(type)
+  {
+    description_ = new char[description.size()+1];
+    strcpy(description_, description.c_str());
+  }
+
   AmplOptionsList::~AmplOptionsList()
   {
     if (keywds_) {
@@ -928,6 +988,8 @@ namespace Ipopt
       nkeywds_ = 0;
     }
 
+    /* PrintLatex(jnlst); */
+
     Index n_options = NumberOfAmplOptions();
     keyword* keywords = new keyword[n_options];
 
@@ -976,6 +1038,59 @@ namespace Ipopt
     return keywds_;
   }
 
+  void AmplOptionsList::MakeValidLatexString(std::string source, std::string& dest) const
+  {
+    std::string::iterator c;
+    for (c=source.begin(); c!=source.end(); c++) {
+      if (*c == '_') {
+        dest.append("\\_");
+      }
+      else if (*c == '^') {
+        dest.append("\\^");
+      }
+      else if (*c == '>' || *c == '<') {
+        ;
+      }
+      else if (*c == '\n') {
+        dest += ", ";
+      }
+      else {
+        dest += *c;
+      }
+    }
+  }
+
+  void AmplOptionsList::PrintLatex(SmartPtr<const Journalist> jnlst)
+  {
+     jnlst->Printf(J_SUMMARY, J_DOCUMENTATION, "\\begin{description}\n");
+     for( std::map<std::string, SmartPtr<const AmplOption> >::iterator iter = ampl_options_map_.begin();
+          iter != ampl_options_map_.end(); ++iter)
+     {
+        std::string amplname;
+        std::string ipoptname;
+        std::string descr;
+
+        MakeValidLatexString(iter->first.c_str(), amplname);
+        MakeValidLatexString(iter->second->IpoptOptionName(), ipoptname);
+        MakeValidLatexString(iter->second->Description(), descr);
+
+        if( ipoptname.length() > 0 )
+        {
+           jnlst->Printf(J_SUMMARY, J_DOCUMENTATION, "\\item[{\\htmlref{%s}{opt:%s}}]",
+              amplname.c_str(), iter->second->IpoptOptionName().c_str());
+           if( amplname != ipoptname )
+              jnlst->Printf(J_SUMMARY, J_DOCUMENTATION, " (Ipopt name: \\htmlref{%s}{opt:%s})",
+                 ipoptname.c_str(), iter->second->IpoptOptionName().c_str());
+        }
+        else
+        {
+           jnlst->Printf(J_SUMMARY, J_DOCUMENTATION, "\\item[%s]", amplname.c_str());
+        }
+        jnlst->Printf(J_SUMMARY, J_DOCUMENTATION, " %s\n", descr.c_str());
+     }
+     jnlst->Printf(J_SUMMARY, J_DOCUMENTATION, "\\end{description}\n");
+  }
+
   char*
   AmplTNLP::get_options(const SmartPtr<OptionsList>& options,
                         SmartPtr<AmplOptionsList>& ampl_options_list,
@@ -997,7 +1112,7 @@ namespace Ipopt
     ampl_options_list->AddAmplOption("outlev",
                                      "print_level",
                                      AmplOptionsList::Integer_Option,
-                                     "Verbosity level (same as print_level)");
+                                     "Verbosity level");
     ampl_options_list->AddAmplOption("print_user_options",
                                      "print_user_options",
                                      AmplOptionsList::String_Option,
@@ -1031,7 +1146,11 @@ namespace Ipopt
     ampl_options_list->AddAmplOption("maxit",
                                      "max_iter",
                                      AmplOptionsList::Integer_Option,
-                                     "Maximum number of iterations (same as max_iter)");
+                                     "Maximum number of iterations");
+    ampl_options_list->AddAmplOption("max_cpu_time",
+                                     "max_cpu_time",
+                                     AmplOptionsList::Number_Option,
+                                     "CPU time limit");
     ampl_options_list->AddAmplOption("compl_inf_tol",
                                      "compl_inf_tol",
                                      AmplOptionsList::Number_Option,
@@ -1227,15 +1346,19 @@ namespace Ipopt
                                      "ma57_pivtolmax",
                                      AmplOptionsList::Number_Option,
                                      "Maximal pivot tolerance for the linear solver MA57");
+    ampl_options_list->AddAmplOption("ma57_pivot_order",
+                                     "ma57_pivot_order",
+                                     AmplOptionsList::Integer_Option,
+                                     "Controls pivot order in MA57");
 
     ampl_options_list->AddAmplOption("pardiso_matching_strategy",
                                      "pardiso_matching_strategy",
                                      AmplOptionsList::String_Option,
                                      "Matching strategy for linear solver Pardiso");
-    ampl_options_list->AddAmplOption("pardiso_out_of_core_power",
-                                     "pardiso_out_of_core_power",
-                                     AmplOptionsList::Integer_Option,
-                                     "Enables out-of-core version of linear solver Pardiso");
+    //ampl_options_list->AddAmplOption("pardiso_out_of_core_power",
+    //                                 "pardiso_out_of_core_power",
+    //                                 AmplOptionsList::Integer_Option,
+    //                                 "Enables out-of-core version of linear solver Pardiso");
 
 #ifdef HAVE_WSMP
 
@@ -1275,7 +1398,7 @@ namespace Ipopt
                                              (void**)&nerror_);
 
     static const char sname_default[] = "ipopt";
-    static const char bsname_default[] = "Ipopt " PACKAGE_VERSION;
+    static const char bsname_default[] = "Ipopt " IPOPT_VERSION;
     static const char opname_default[] = "ipopt_options";
     const char* sname;
     const char* bsname;
@@ -1435,6 +1558,21 @@ namespace Ipopt
     return dp->u.i;
   }
 
+  std::vector<Index>
+  AmplSuffixHandler::GetIntegerSuffixValues(Index n, std::string suffix_string,
+      Suffix_Source source) const
+  {
+    std::vector<Index> ret;
+    const Index* ptr = GetIntegerSuffixValues(suffix_string, source);
+    if (ptr) {
+      ret.reserve(n);
+      for (Index i=0; i<n; i++) {
+        ret.push_back(ptr[i]);
+      }
+    }
+    return ret;
+  }
+
   const Number*
   AmplSuffixHandler::GetNumberSuffixValues(std::string suffix_string,
       Suffix_Source source) const
@@ -1462,6 +1600,22 @@ namespace Ipopt
     SufDesc* dp = suf_get(suffix_string.c_str(), kind);
     return dp->u.r;
   }
+
+  std::vector<Number>
+  AmplSuffixHandler::GetNumberSuffixValues(Index n, std::string suffix_string,
+      Suffix_Source source) const
+  {
+    std::vector<Number> ret;
+    const Number* ptr = GetNumberSuffixValues(suffix_string, source);
+    if (ptr) {
+      ret.reserve(n);
+      for (Index i=0; i<n; i++) {
+        ret.push_back(ptr[i]);
+      }
+    }
+    return ret;
+  }
+
 } // namespace Ipopt
 
 
